(* This module handles authentication by means of a user-editable cookie; this
   should be used for testing purposes only! Any 'real' authentication module
   should have a similar interface. *)
con user = [Username = string, Teacher = bool]

cookie userCookie : string

table userTbl : user PRIMARY KEY (Username)

(* Get the current user, if any. *)
val getUser : transaction (option $user)
(* Get the list of all users. *)
val getUsers : transaction (list $user)
(* Get the list of all students. *)
val getStudents : transaction (list $user)
(* Perform some action with the current user; errors if there is none. *)
val withUser : a ::: Type -> ($user -> transaction a) -> transaction a
(* Errors if the client is not logged in or the user is not a teacher. *)
val requireTeacher : transaction unit
(* Errors if the client is not logged in. *)
val requireLogin : transaction unit
(* Require that the current user be a teacher or have the specified username. *)
val requireTeacherOr : string -> transaction unit
(* Include some XML if the current user is a teacher. *)
val ifTeacher : xbody -> transaction xbody
(* Include some XML if the current user is a student. *)
val ifStudent : xbody -> transaction xbody
