open Authentication
open Bootstrap3
open Template

val template = template

structure BM = BasicMaterialModule
structure RQ = ReadingQuestionModule
structure HW = HomeworkModule
structure RM = ReadingMaterialModule
structure Q = QuizModule

fun makeCalItems (linker : Gradable.id -> transaction page) [extra ::: {Type}]
                 [extra ~ Gradable.gradableCols ++ Material.cols]
                 (gradables : list $(Gradable.gradableCols ++ Material.cols ++
                                                           extra))
    : list CalendarModule.item = List.mp (fn gradable =>
      { Title = gradable.Title,
        Datetime = Datetime.fromTime gradable.DueDate,
        Link = url (linker gradable.Id) }) gradables

structure Cal = CalendarModule.Make(struct
    val getItems =
        readingQuestions <- Monad.mp (makeCalItems RQ.viewOne) RQ.Grad.list;
        homework <- Monad.mp (makeCalItems HW.viewOne) HW.Grad.list;
        quizzes <- Monad.mp (makeCalItems Q.viewOne) Q.Grad.list;
        return (Util.concat (readingQuestions :: homework :: quizzes :: []))
    val template = template
  end)

fun gradebookLinker linker lister user : transaction (list Gradebook.item) =
    listed <- lister user;
    return (List.mp (fn x =>
                        (x ++ {Url = url (linker x.Gradable)}) -- #Gradable)
                    listed)

structure GB = Gradebook.Make(struct
    val gradableInfos = {
        Homework = {
        Label = "Homework",
        ListItems = gradebookLinker HW.viewOne HW.Grad.gradeSummary,
        Weight = 40
        },
        Quizzes = {
        Label = "Quizzes",
        ListItems = gradebookLinker Q.viewOne Q.Grad.gradeSummary,
        Weight = 80
        }
    }
    end)

val main = fn _ => template { Inner = <xml>
  Click one of the links above to use the corresponding module.
</xml>, Crumbs = [] }

(* This is kind of a hack; it's necessary so that Ur/Web generates the relevant
   handlers, since they're not mentioned except by template. *)
val linker = template { Inner = <xml>
  <a link={BM.viewAll ()}></a>
  <a link={RQ.viewAll ()}></a>
  <a link={HW.viewAll ()}></a>
  <a link={RM.viewAll ()}></a>
  <a link={Q.viewAll ()}></a>
  <a link={Cal.currentCalendar ()}></a>
  <a link={GB.showGrades ()}></a>
</xml>, Crumbs = [] }

fun setUserCookie r =
    setCookie userCookie {Value = r.Username, Secure = False, Expires = None};
    editUserCookie ()

and editUserCookie () =
    u <- getCookie userCookie;
    return <xml><body>
      {case u of
           None => <xml>No user set</xml>
         | Some u' => <xml>
           Cookie: Username = {[u']}
           </xml>
      }

      <form>
        <div>Username: <textbox{#Username}/></div>
        <div><submit action={setUserCookie}/></div>
      </form>
    </body></xml>
