con id = Material.id

(* A Gradable can have multiple components (e.g., parts of a problem set, quiz
   questions). *)
con component = [Id = int, Gradable = serialized id, Index = int,
                    Description = string, MaxGrade = int]

(* This stores the information on what grade a student has for a given
   component, which is separate from their submission for that assignment. *)
con gradeCols = [Component = int, Username = string, Grade = int]

(* Information about a submission: what it was a submission for, who submitted
   it, and when. *)
con submissionCols = [Gradable = serialized id, Username = string,
                      Submitted = time]
(* Information to add before passing to Material. *)
con gradableCols = [DueDate = time, Weight = int]

(* All routes that a Gradable exposes. *)
con routes = Material.routes ++ [Submit = id, Components = id, Grades = id,
                                 ViewSubmission = id * string]

functor Make(M : sig
    con submissionData :: {Type}
    constraint submissionData ~ submissionCols
    val submissionFl : folder submissionData
    val submissionInj : $(map sql_injectable submissionData)
    val validateSubmission : $submissionData -> transaction (Util.result {})

    con gradableData :: {Type}
    val gradableFl : folder gradableData
    constraint Material.cols ~ gradableData
    constraint gradableCols ~ gradableData
    val validateGradable : $gradableData -> transaction (Util.result {})
  end) : sig
    con t' = [Title = string] ++ gradableCols ++ M.gradableData
    con t = [Id = id] ++ t'

    (* Create a new gradable. *)
    val create : $t' -> transaction (Util.result $t)
    (* Look up a gradable by ID. *)
    val lookup : id -> transaction (option $t)
    (* Update the data for a gradable. *)
    val update : id -> $t -> transaction (Util.result {})
    (* Delete a gradable; does nothing if there is no gradable with that id. *)
    val delete : id -> transaction unit
    (* List all gradables. *)
    val list : transaction (list $t)

    (* Produce a summary of a student's grades in a format suitable for the
       gradebook. The Grade field is the sum of all grades received; the
       MaxGrade is the sum of all maximum grades for graded assignments. If no
       assignments are graded then both are None. *)
    val gradeSummary : $Authentication.user ->
                       transaction (list {Title : string, Gradable : id,
                                          Grade : option int,
                                          MaxGrade : option int,
                                          Weight : int})

    functor Gui(R : sig
        (* The name of this kind of gradable. *)
        val name : string
        con auxRoutes :: {Type}
        val auxRoutesFl : folder auxRoutes
        constraint routes ~ auxRoutes
        val auxHandlers : Linkage.handlers (routes ++ auxRoutes) auxRoutes
        val viewGradable : Linkage.router (routes ++ auxRoutes) ->
                           option id -> $M.gradableData -> transaction xbody
        val viewSubmission : Linkage.router (routes ++ auxRoutes) ->
                             option id ->
                             $(M.submissionData ++ submissionCols) ->
                             transaction xbody
        con formState :: Type
        val formEls : Gui.formElement (routes ++ auxRoutes)
                                      (option id) M.gradableData formState body
        type submissionState
        val submissionEl : Gui.formElement (routes ++ auxRoutes)
                           (option $submissionCols)
                           M.submissionData submissionState body
      end) : sig
        val routed : $(map (fn t => t -> transaction page)
                           (routes ++ R.auxRoutes))
    end
end
