open Authentication
open Bootstrap3

type item = {Title : string, Grade : option int, MaxGrade : option int,
             Weight : int, Url : url}
type gradableInfo = {Label : string,
                     ListItems : $user -> transaction (list item),
                     Weight : int}

style grade_block

functor Make(M : sig
    con gradables :: {Unit}
    val fl : folder gradables
    val gradableInfos : $(mapU gradableInfo gradables)
end) = struct

    (* This builds the set of rows for an individual gradable module's items,
       keeping track of the total grade as well as the max grade for every
       graded item. *)
    fun gradableBuilder item acc = let
        val pct =
            grade <- item.Grade;
            maxGrade <- item.MaxGrade;
            return (100.0 * float grade / float maxGrade)
    in {
        Xml = join acc.Xml <xml><tr>
          <td><a href={item.Url}>{[item.Title]}</a></td>
          <td>{[Option.mp trunc pct]}</td>
        </tr>
        </xml>,
        WeightedSum = acc.WeightedSum + float item.Weight * Option.get 0.0 pct,
        TotalWeight = acc.TotalWeight + (if Option.isSome pct
                                         then float item.Weight else 0.0)
    }
    end

    fun gradebookBuilder (info : gradableInfo) acc =
        withUser (fn user =>
            items <- info.ListItems user;
            let
                val built = List.foldl gradableBuilder {
                    Xml = <xml><tr><th colspan=4>{[info.Label]}</th></tr></xml>,
                    WeightedSum = 0.0,
                    TotalWeight = 0.0
                } items
                val percent = built.WeightedSum / built.TotalWeight
            in
                return {
                Xml = <xml>
                  <tbody class="grade-block">
                  {built.Xml}
                  <tr>
                    <th>Total</th>
                    <td>
                      {[if built.TotalWeight = 0.0 then ""
                        else show (trunc percent)]}
                    </td>
                  </tr>
                  </tbody>
                  {acc.Xml}
                </xml>,
                WeightedSum = percent * float info.Weight + acc.WeightedSum,
                TotalWeight = (if built.TotalWeight = 0.0 then 0.0
                               else float info.Weight) + acc.TotalWeight
                }
            end)

    fun showGrades () =
        folded <-
        @foldUR [gradableInfo] [fn _ => transaction {Xml : xtable,
                                                     WeightedSum : float,
                                                     TotalWeight : float}]
         (fn [nm ::_] [rest ::_] [[nm] ~ rest] info acc =>
             acc' <- acc; gradebookBuilder info acc')
         (return {Xml = <xml/>, WeightedSum = 0.0, TotalWeight = 0.0}) M.fl
         M.gradableInfos;
        Template.template {
        Inner = <xml><table class="bs3-table">
          {folded.Xml}
          <tbody>
            <tr>
              <th>Grand Total</th>
              <td>
                {[if folded.TotalWeight = 0.0 then ""
                  else show (trunc (folded.WeightedSum / folded.TotalWeight))]}
            </td>
            </tr>
          </tbody>
        </table></xml>,
        Crumbs = {Url = None, Title = "Gradebook"} :: []}
end
