(* The type arguments are the routing information, the type of 'metadata' (used
   for initialization) a record of data values, the 'internal state', and the
   type of the rendered XML. *)

con formElement = fn (routes :: {Type}) (meta :: Type) (fs :: {Type})
                                        (state :: Type) (ctx :: {Unit}) => {
                   Init : Linkage.router routes -> meta -> option $fs ->
                          transaction state,
                   Signals : state -> signal (Util.result $fs),
                   Edit : state -> xml ctx [] [],
                   OnSave : state -> transaction unit,
                   Validate : $fs -> transaction (Util.result {})
                 }

(* Given a string input source, a parser, and a validator, create a signal that
   represents the parsed and validated data. *)
val dataSignal : a ::: Type -> source string -> (string -> Util.result a) ->
                 (a -> option string) -> signal (Util.result a)
(* Turn a list (result x) into an result (list x). Haskell calls this
   'sequence', but that's a keyword. There's no reason this can't be
   polymorphic, but that causes mysterious errors along the lines of
   'polymorphic type needed too late.' *)
val perform : a ::: Type -> list (Util.result a) -> Util.result (list a)

val textboxEl : routes ::: {Type} -> a ::: Type -> nm :: Name -> string ->
                string -> (string -> option string) ->
                formElement routes a [nm = string] (source string) body
val textareaEl : routes ::: {Type} -> a ::: Type -> nm :: Name -> string ->
                 string -> (string -> option string) ->
                 formElement routes a [nm = string] (source string) body
val intEl : routes ::: {Type} -> a ::: Type -> nm :: Name -> string -> int ->
            (int -> option string) ->
            formElement routes a [nm = int] (source string) body
val dateEl : routes ::: {Type} -> a ::: Type -> nm :: Name -> string ->
             Datetime.t -> (Datetime.t -> option string) ->
             formElement routes a [nm = time] DatetimePicker.t body
val xmlEl : routes ::: {Type} -> a ::: Type ->
            (Linkage.router routes -> a -> transaction xbody) ->
            formElement routes a [] xbody body

(* This represents the data needed per column of a table. *)
con recordRowData = fn t => {
    (* A default value for the column. *)
    Default : t,
    Stringify : t -> string,
    Parse : string -> Util.result t,
    (* This should return None if there is no error, or Some if there is. *)
    Validate : t -> Util.result {},
}

val tableEl : routes ::: {Type} -> nm :: Name -> meta ::: Type ->
              fs ::: {Type} -> state ::: Type -> bool ->
              formElement routes meta fs state tr ->
              formElement routes {Headers : list xbody, Meta : list meta,
                                  Default : meta}
                          [nm = list $fs]
                          {Router : Linkage.router routes,
                           Headers : list xbody,
                           DefaultMeta : meta,
                           States : source (list state)} body

val recordRow : routes ::: {Type} -> meta ::: Type -> fs ::: {Type} ->
                folder fs -> $(map recordRowData fs) ->
                formElement routes meta fs
                            $(map (fn _ => source string) fs) tr

functor Upload(M : sig
               end) : sig
    val getUpload : int -> transaction (option {MimeType : string,
                                                Content : blob})
    val uploadEl : uploadRoute :: Name -> routes ::: {Type} ->
                   [[uploadRoute] ~ routes] => a ::: Type -> nm :: Name ->
                   bool ->
                   formElement (routes ++ [uploadRoute = int]) a
                               [nm = option int]
                               {IdSrc : source (option int),
                                HandleSrc : source (option AjaxUpload.handle),
                                UploaderXml : xbody} body
end

val concatForm : routes ::: {Type} -> meta ::: Type -> fs1 ::: {Type} ->
                 state1 ::: Type -> fs2 ::: {Type} -> state2 ::: Type ->
                 [fs1 ~ fs2] => ctx ::: {Unit} ->
                 formElement routes meta fs1 state1 ctx ->
                 formElement routes meta fs2 state2 ctx ->
                 formElement routes meta (fs1 ++ fs2) (state1 * state2) ctx

val usingMeta : routes ::: {Type} -> meta ::: Type -> meta' ::: Type ->
                fs ::: {Type} -> state ::: Type -> ctx ::: {Unit} ->
                (meta' -> meta) ->
                formElement routes meta fs state ctx ->
                formElement routes meta' fs state ctx
