(* A router that takes a variant. *)
type router (requires :: {Type}) = variant requires -> url

(* A single handler that requires a record of handlers that it might link to, as
   well as a parameter. Passing multiple arguments can be accomplished by using
   a tuple or record. *)
type handler (requires :: {Type}) (param :: Type) =
     router requires -> param -> transaction page

(* A record of handlers, indicating both what it requires and what it
   provides. Note that the [provides] parameter can include fields in the
   [requires] parameter; this happens when two mutually recursive handler
   records are linked together.  *)
type handlers (requires :: {Type}) (provides :: {Type}) =
     $(map (handler requires) provides)

(* A self-sufficient record of handlers. *)
type closable (linkage :: {Type}) = handlers linkage linkage

(* "Tie the knot" of a group of handlers. If [provides = [Foo = int]], then the
   handlers can be accessed via, e.g., [handlers.Foo 8]. *)
functor Close(M : sig
                  con provides :: {Type}
                  val fl : folder provides
                  val fs : closable provides
              end) : sig
    val routed: $(map (fn t => t -> transaction page) M.provides)
end

(* Weaken the requirements of a handler; it's always valid to require more
   things.*)
val weaken : requires ::: {Type} -> unused ::: {Type} -> provides ::: {Type}
             -> [requires ~ unused] => folder provides -> folder requires
             -> handlers requires provides
             -> handlers (requires ++ unused) provides
