(* Quizzes are administered in-person, so students don't submit them. Instead,
   the grades are entered by the teacher(s), who can also upload the quiz as
   well as the solution. *)

structure QuizUpload = Gui.Upload(struct end)
structure SolutionUpload = Gui.Upload(struct end)

fun getQuizUpload uploadId =
    Authentication.requireLogin;
    upload <- QuizUpload.getUpload uploadId;
    case upload of
        None => error <xml>404!</xml>
      | Some upload' => returnBlob upload'.Content (blessMime upload'.MimeType)

fun getSolutionUpload uploadId : transaction page =
    Authentication.requireLogin;
    upload <- SolutionUpload.getUpload uploadId;
    case upload of
        None => error <xml>404!</xml>
      | Some upload' => returnBlob upload'.Content (blessMime upload'.MimeType)

val quizEl = Gui.concatForm
                 (Gui.textareaEl [#Description] "Description" "" Util.nonEmpty)
                 (Gui.concatForm
                      (QuizUpload.uploadEl [#ViewQuiz] [#Quiz] False)
                      (SolutionUpload.uploadEl [#ViewSolution] [#Solution]
                                               False))
val submissionEl = Gui.xmlEl (fn _ _ => return <xml/>)

structure Grad = Gradable.Make(struct
    con submissionData = []
    val validateGradable = quizEl.Validate
    val validateSubmission = submissionEl.Validate
  end)

structure Gui = Grad.Gui(struct
    val name = "Quiz"
    val viewGradable = fn r _ quiz => return <xml>
      <h2>Description</h2>
      <p>{[quiz.Description]}</p>
      {case quiz.Quiz of
           Some id => <xml>
             <p><a href={r (make [#ViewQuiz] id)}>Solution</a></p>
           </xml>
         | None => <xml/>}
      {case quiz.Solution of
            Some id => <xml>
              <p><a href={r (make [#ViewSolution] id)}>Solution</a></p>
            </xml>
          | None => <xml/>}
         </xml>
     val viewSubmission = fn _ _ _ => return <xml/>
     val formEls = quizEl
     val auxHandlers = {ViewQuiz = fn r id => getQuizUpload id,
                        ViewSolution = fn r id => getSolutionUpload id}
     val submissionEl = submissionEl
  end)

val viewOne = Gui.routed.ViewOne
val viewAll = Gui.routed.ViewAll
