(* This represents a reading question that students are expected to respond to
   before class. The question can be answered directly on the website itself. *)

val gradableEl =  Gui.textboxEl [#Question] "Question"
                                "What is the meaning of life?" Util.nonEmpty
val submissionEl =  let
        val responseEl = Gui.textboxEl [#Response] "Response" "" Util.nonEmpty
        val classQuestionEl = Gui.textboxEl [#ClassQuestion] "Class question" ""
                                            Util.nonEmpty
    in
        Gui.concatForm responseEl classQuestionEl
    end

structure Grad = Gradable.Make(struct
    con submissionData = [Response = string, ClassQuestion = string]
    val validateGradable = gradableEl.Validate
    val validateSubmission = submissionEl.Validate
  end)

structure Gui = Grad.Gui(struct
    val name = "Reading Question"
    val viewGradable = fn _ _ question => return <xml>
      <h2>Question</h2>
      <div>{[question.Question]}</div>
    </xml>
    val viewSubmission = fn _ _ submission => return <xml>
      <h2>Response</h2><div>{[submission.Response]}</div>
      <h2>Class question</h2><div>{[submission.ClassQuestion]}</div>
    </xml>
    val formEls = gradableEl
    val auxHandlers = {}
    val submissionEl = submissionEl
  end)

val viewOne = Gui.routed.ViewOne
val viewAll = Gui.routed.ViewAll
