open Bootstrap3

type t = { Inner : xbody, Crumbs : list ({Url : option url, Title : string}) }

val activ = Basis.active

fun breadcrumbs crumbs : xbody = case crumbs of
  [] => <xml/>
| _ => let
      val n = List.length crumbs
  in
      <xml>
        <ol class="breadcrumb">
          {List.mapXi (fn i crumb => <xml>
            <li class={if i = (n - 1) then Bootstrap3.bs3_active else null}>
              {case crumb.Url of
                   None => cdata crumb.Title
                 | Some url => <xml><a href={url}>{[crumb.Title]}</a></xml>}
          </li></xml>) crumbs}
        </ol>
      </xml>
  end


val navbarItems = {Url = bless "/BasicMaterialModule/viewAll",
                   Title = "Basic Material"} ::
                  {Url = bless "/ReadingQuestionModule/viewAll",
                   Title = "Reading Question"} ::
                  {Url = bless "/HomeworkModule/viewAll",
                   Title = "Homework"} ::
                  {Url = bless "/ReadingMaterialModule/viewAll",
                   Title = "Reading Material"} ::
                  {Url = bless "/QuizModule/viewAll",
                   Title = "Quizzes"} ::
                  {Url = bless "/Galactic/Cal/currentCalendar",
                   Title = "Calendar"} ::
                  {Url = bless "/Galactic/GB/showGrades",
                   Title = "Gradebook"} :: []

val navbar : xbody = <xml>
  <div class="navbar navbar-default">
    <div class="container">
      <div class="navbar_header">
        <a class="navbar_brand">Galactic</a>
      </div>
      <div class="collapse navbar-collapse">
        <ul class="nav navbar-nav">
          {List.mapX (fn item => <xml><li>
            <a href={item.Url}>{[item.Title]}</a>
          </li></xml>) navbarItems}
        </ul>
      </div>
    </div>
  </div>
</xml>

fun template (data : t) : transaction page =
    n <- Datetime.now;
    picker <- DatetimePicker.create (Some n);
    return
    <xml>
      <head>
        <title>The Galactic Course Management System</title>
        <link rel="stylesheet" href="http://netdna.bootstrapcdn.com/bootstrap/3.0.0/css/bootstrap.min.css"/>
        <link rel="stylesheet" href="http://web.mit.edu/~phurst/Public/galactic/css/galactic.css"/>
        <link rel="stylesheet" href="http://web.mit.edu/~phurst/Public/galactic/css/bootstrap-datetimepicker.min.css"/>
      </head>
      <body>
        {navbar}
        <div class="container">
          {breadcrumbs data.Crumbs}
          {data.Inner}
        </div>
      </body>
    </xml>
