(* Random utility functions. *)

(* A datatype for representing a result of an operation that can fail. *)
datatype result t = Error of string | Success of t
val result_monad = mkMonad {
                   Return = fn [t] x => Success x,
                   Bind = fn [t1] [t2] x f => case x of
                                                  Error err => Error err
                                                | Success x' => f x'
                   }
fun getError [t ::: Type] (x : result t) : xbody = case x of
                     Error err => cdata err
                   | Success _ => <xml/>
fun isSuccess [t ::: Type] (x : result t) : bool = case x of
                     Error _ => False
                   | Success _ => True



(* Get a field from an option value with default. *)
fun getOptField [nm :: Name] [a ::: Type] (v : a) [fs ::: {Type}] [fs ~ [nm]]
                (x : option $(fs ++ [nm = a])) : a =
                                                 case x of
                                                     None => v
                                                   | Some x' => x'.nm

fun setField [nm :: Name] [a ::: Type] (x : a) [fs ::: {Type}] [[nm] ~ fs]
             (v : $(fs ++ [nm = a])) = (v -- nm) ++ {nm = x}

fun nonEmpty str = if str = "" then Some "Field must not be empty" else None

fun listPairMapM [m] (_ : monad m) [a] [b] [c] (f : a -> b -> m c)
    : list a -> list b -> m (list c) = let
    fun mapM' xs ys =
        case (xs, ys) of
            ([], []) => return []
          | (x :: xs', y :: ys') =>
            z <- f x y;
            zs <- mapM' xs' ys';
            return (z :: zs)
          | _ => error <xml>listPairMapM: Unequal list lengths</xml>
in
    mapM'
end

fun concat [a] (xs : list (list a)) : list a =
    case xs of
        [] => []
      | (x :: xs) => List.append x (concat xs)

fun project [fs :: {Type}] [rest ::: {Type}] [fs ~ rest] (x : $(fs ++ rest)) =
    x --- rest
